@@ -221,14 +221,18 @@ class AutoSchema(ViewInspector):
                 parameter = force_instance(parameter)
                 mapped = self._map_serializer(parameter, 'request')
                 for property_name, property_schema in mapped['properties'].items():
-                    field = parameter.fields.get(property_name)
+                    try:
+                        # be graceful when serializer might be non-DRF (via extension).
+                        field = parameter.fields.get(property_name)
+                    except Exception:
+                        field = None
                     result[property_name, OpenApiParameter.QUERY] = build_parameter_type(
                         name=property_name,
                         schema=property_schema,
                         description=property_schema.pop('description', None),
                         location=OpenApiParameter.QUERY,
                         allow_blank=getattr(field, 'allow_blank', True),
-                        required=field.required,
+                        required=bool(property_name in mapped.get('required', [])),
                     )
             else:
                 warn(f'could not resolve parameter annotation {parameter}. Skipping.')
@@ -460,7 +464,12 @@ class AutoSchema(ViewInspector):
         if re.search(r'<drf_format_suffix\w*:\w+>', self.path_regex):
             tokenized_path.append('formatted')
 
-        return '_'.join(tokenized_path + [action])
+        if spectacular_settings.OPERATION_ID_METHOD_POSITION == 'PRE':
+            return '_'.join([action] + tokenized_path)
+        elif spectacular_settings.OPERATION_ID_METHOD_POSITION == 'POST':
+            return '_'.join(tokenized_path + [action])
+        else:
+            assert False, 'Invalid value for OPERATION_ID_METHOD_POSITION. Allowed: PRE, POST'
 
     def is_deprecated(self) -> bool:
         """ override this for custom behaviour """
@@ -690,7 +699,10 @@ class AutoSchema(ViewInspector):
             # read_only fields do not have a Manager by design. go around and get field
             # from parent. also avoid calling Manager. __bool__ as it might be customized
             # to hit the database.
-            if getattr(field, 'queryset', None) is not None:
+            if not is_slug and getattr(field, 'pk_field') is not None:
+                schema = self._map_serializer_field(field.pk_field, direction)
+                return append_meta(schema, meta)
+            elif getattr(field, 'queryset', None) is not None:
                 if is_slug:
                     model = field.queryset.model
                     source = [field.slug_field]
@@ -709,7 +721,7 @@ class AutoSchema(ViewInspector):
                         f'Could not derive type for under-specified {field.__class__.__name__} '
                         f'"{field.field_name}". The serializer has no associated model (Meta class) '
                         f'and this particular field has no type without a model association. Consider '
-                        f'changing the field or adding a Meta class. defaulting to string.'
+                        f'changing the field or adding a Meta class. Defaulting to string.'
                     )
                     return append_meta(build_basic_type(OpenApiTypes.STR), meta)
 
@@ -893,7 +905,14 @@ class AutoSchema(ViewInspector):
                 return append_meta(build_basic_type(OpenApiTypes.STR), meta)
 
             target = follow_field_source(model, source)
-            if callable(target):
+            if (
+                hasattr(target, "_partialmethod")
+                and target._partialmethod.func.__name__ == '_get_FIELD_display'
+            ):
+                # the only way to detect an uninitialized partial method
+                # this is a convenience method for model choice fields and is mostly a string
+                schema = build_basic_type(str)
+            elif callable(target):
                 schema = self._map_response_type_hint(target)
             elif isinstance(target, models.Field):
                 schema = self._map_model_field(target, direction)
@@ -1057,6 +1076,10 @@ class AutoSchema(ViewInspector):
     def _insert_field_validators(self, field, schema):
         schema_type = schema.get('type')
 
+        # OAS 3.1 special case - extract the main type
+        if isinstance(schema_type, list):
+            schema_type = [t for t in schema_type if t != 'null'][0]
+
         def update_constraint(schema, key, function, value, *, exclusive=False):
             if callable(value):
                 value = value()
@@ -1471,12 +1494,13 @@ class AutoSchema(ViewInspector):
                 and is_serializer(serializer)
                 and (not is_list_serializer(serializer) or is_serializer(serializer.child))
             ):
-                paginated_name = self.get_paginated_name(self._get_serializer_name(serializer, "response"))
                 component = ResolvedComponent(
-                    name=paginated_name,
+                    name=self.get_paginated_name(self._get_serializer_name(serializer, 'response')),
                     type=ResolvedComponent.SCHEMA,
                     schema=paginator.get_paginated_response_schema(schema),
-                    object=serializer.child if is_list_serializer(serializer) else serializer,
+                    object=self.get_serializer_identity(
+                        serializer.child if is_list_serializer(serializer) else serializer, 'response'
+                    )
                 )
                 self.registry.register_on_missing(component)
                 schema = component.ref
@@ -1549,7 +1573,17 @@ class AutoSchema(ViewInspector):
 
         return result
 
+    def get_serializer_identity(self, serializer, direction: Direction) -> Any:
+        serializer_extension = OpenApiSerializerExtension.get_match(serializer)
+        if serializer_extension:
+            identity = serializer_extension.get_identity(self, direction)
+            if identity is not None:
+                return identity
+
+        return serializer
+
     def get_serializer_name(self, serializer: serializers.Serializer, direction: Direction) -> str:
+        """ override this for custom behaviour """
         return serializer.__class__.__name__
 
     def _get_serializer_name(self, serializer, direction, bypass_extensions=False) -> str:
@@ -1605,7 +1639,7 @@ class AutoSchema(ViewInspector):
             component = ResolvedComponent(
                 name=self._get_serializer_name(serializer, direction, bypass_extensions),
                 type=ResolvedComponent.SCHEMA,
-                object=serializer,
+                object=self.get_serializer_identity(serializer, direction),
             )
             if component in self.registry:
                 return self.registry[component]  # return component with schema
