@@ -420,8 +420,13 @@ def build_parameter_type(
 def build_choice_field(field) -> _SchemaType:
     choices = list(OrderedDict.fromkeys(field.choices))  # preserve order and remove duplicates
 
-    if all(isinstance(choice, bool) for choice in choices):
-        type: Optional[str] = 'boolean'
+    if field.allow_blank and '' not in choices:
+        choices.append('')
+
+    if not choices:
+        type = None
+    elif all(isinstance(choice, bool) for choice in choices):
+        type = 'boolean'
     elif all(isinstance(choice, int) for choice in choices):
         type = 'integer'
     elif all(isinstance(choice, (int, float, Decimal)) for choice in choices):  # `number` includes `integer`
@@ -432,8 +437,6 @@ def build_choice_field(field) -> _SchemaType:
     else:
         type = None
 
-    if field.allow_blank and '' not in choices:
-        choices.append('')
     if field.allow_null and None not in choices:
         choices.append(None)
 
@@ -536,12 +539,18 @@ def safe_ref(schema: _SchemaType) -> _SchemaType:
 
 def append_meta(schema: _SchemaType, meta: _SchemaType) -> _SchemaType:
     if spectacular_settings.OAS_VERSION.startswith('3.1'):
+        schema = schema.copy()
+        meta = meta.copy()
+
         schema_nullable = meta.pop('nullable', None)
         meta_nullable = schema.pop('nullable', None)
 
         if schema_nullable or meta_nullable:
             if 'type' in schema:
-                schema['type'] = [schema['type'], 'null']
+                if isinstance(schema['type'], str):
+                    schema['type'] = [schema['type'], 'null']
+                else:
+                    schema['type'] = [*schema['type'], 'null']
             elif '$ref' in schema:
                 schema = {'oneOf': [schema, {'type': 'null'}]}
             elif len(schema) == 1 and 'oneOf' in schema:
@@ -723,6 +732,17 @@ class ResolvedComponent:
         return {'$ref': f'#/components/{self.type}/{self.name}'}
 
 
+class ComponentIdentity:
+    """ A container class to make object/component comparison explicit """
+    def __init__(self, obj):
+        self.obj = obj
+
+    def __eq__(self, other):
+        if isinstance(other, ComponentIdentity):
+            return self.obj == other.obj
+        return self.obj == other
+
+
 class ComponentRegistry:
     def __init__(self) -> None:
         self._components: Dict[Tuple[str, str], ResolvedComponent] = {}
@@ -746,17 +766,25 @@ class ComponentRegistry:
 
         query_obj = component.object
         registry_obj = self._components[component.key].object
-        query_class = query_obj if inspect.isclass(query_obj) else query_obj.__class__
-        registry_class = query_obj if inspect.isclass(registry_obj) else registry_obj.__class__
+
+        if isinstance(query_obj, ComponentIdentity) or inspect.isclass(query_obj):
+            query_id = query_obj
+        else:
+            query_id = query_obj.__class__
+
+        if isinstance(registry_obj, ComponentIdentity) or inspect.isclass(registry_obj):
+            registry_id = registry_obj
+        else:
+            registry_id = registry_obj.__class__
 
         suppress_collision_warning = (
-            get_override(registry_class, 'suppress_collision_warning', False)
-            or get_override(query_class, 'suppress_collision_warning', False)
+            get_override(registry_id, 'suppress_collision_warning', False)
+            or get_override(query_id, 'suppress_collision_warning', False)
         )
-        if query_class != registry_class and not suppress_collision_warning:
+        if query_id != registry_id and not suppress_collision_warning:
             warn(
                 f'Encountered 2 components with identical names "{component.name}" and '
-                f'different classes {query_class} and {registry_class}. This will very '
+                f'different identities {query_id} and {registry_id}. This will very '
                 f'likely result in an incorrect schema. Try renaming one.'
             )
         return True
@@ -1208,7 +1236,7 @@ def sanitize_result_object(result):
 
 
 def sanitize_specification_extensions(extensions):
-    # https://spec.openapis.org/oas/v3.0.3#specificationExtensions
+    # https://spec.openapis.org/oas/v3.0.3#specification-extensions
     output = {}
     for key, value in extensions.items():
         if not re.match(r'^x-', key):
@@ -1255,6 +1283,7 @@ def build_mock_request(method, path, view, original_request, **kwargs):
 
 def set_query_parameters(url, **kwargs) -> str:
     """ deconstruct url, safely attach query parameters in kwargs, and serialize again """
+    url = str(url)  # Force evaluation of reverse_lazy urls
     scheme, netloc, path, params, query, fragment = urllib.parse.urlparse(url)
     query = urllib.parse.parse_qs(query)
     query.update({k: v for k, v in kwargs.items() if v is not None})
@@ -1263,6 +1292,7 @@ def set_query_parameters(url, **kwargs) -> str:
 
 
 def get_relative_url(url: str) -> str:
+    url = str(url)  # Force evaluation of reverse_lazy urls
     scheme, netloc, path, params, query, fragment = urllib.parse.urlparse(url)
     return urllib.parse.urlunparse(('', '', path, params, query, fragment))
 
@@ -1416,11 +1446,19 @@ def build_listed_example_value(value: Any, paginator, direction):
     if schema is sentinel:
         return [value]
 
+    def drilldown_schema_example(schema, sentinel):
+        # Recursively travel schema properties to build example.
+        if schema is sentinel:
+            return [value]
+        if 'properties' in schema.keys():
+            return {
+                field_name: drilldown_schema_example(field_schema, sentinel)
+                for field_name, field_schema in schema['properties'].items()
+            }
+        return schema['example']
+
     try:
-        return {
-            field_name: [value] if field_schema is sentinel else field_schema['example']
-            for field_name, field_schema in schema['properties'].items()
-        }
+        return drilldown_schema_example(schema, sentinel)
     except (AttributeError, KeyError):
         warn(
             f"OpenApiExample could not be paginated because {paginator.__class__} either "
